import MyTaskModel from "./Task";
import "./StatusColumn";
import "./GanttToolbar";
import "./ResourceGrid";
import { Toast } from "@bryntum/gantt";
import { ProjectModel } from "@bryntum/gantt";

let { configGantt } = require("./GanttCommon");

export const project = new ProjectModel({
  listeners: {
    change() {
      if (configGantt.state.isLoaded) {
        const itemSave = configGantt.state.gantt.tbar.items.find((obj) => {
          return obj.initialConfig.ref === "buttonGroupSaveReload";
        });
        if (itemSave != null) {
          itemSave.enable();
        }
      }
    },
    load() {
      try {
        configGantt.state.isLoaded = true;
      } catch (ex) {
        //ignore ex
      }
    },
  },
  taskModelClass: MyTaskModel,
  resourceStore: {
    tree: true,
  },
  transport: {
    load: {
      url: "/gantt-data.json",
    },
  },
  autoLoad: true,
  taskStore: {
    wbsMode: "auto",
  },
  stm: {
    autoRecord: true,
  },
  resetUndoRedoQueuesAfterLoad: true,
  validateResponse: true,
});

export const containerConfig = {
  items: [
    {
      type: "gantt",
      ref: "gantt",
      dependencyIdField: "wbsCode",
      project: project,
      height: "100%",
      width: "85%",
      startDate: "2022-03-12",
      endDate: "2022-06-24",
      resourceImageFolderPath: "/images/users",
      scrollTaskIntoViewOnCellClick: true,
      subGridConfigs: {
        locked: {
          flex: 3,
        },
        normal: {
          flex: 4,
        },
      },
      columns: [
        { type: "wbs" },
        { type: "name", width: 250 },
        { type: "startdate" },
        { type: "duration" },
        { type: "resourceassignment", width: 120, showAvatars: true },
        { type: "percentdone", showCircle: true, width: 70 },
        {
          type: "predecessor",
          width: 112,
        },
        {
          type: "successor",
          width: 112,
        },
        { type: "schedulingmodecolumn" },
        { type: "calendar" },
        { type: "constrainttype" },
        { type: "constraintdate" },
        { type: "statuscolumn" },
        {
          type: "date",
          text: "Deadline",
          field: "deadline",
        },
        { type: "addnew" },
      ],
      columnLines: false,
      features: {
        mspExport: true, // enable the feature Microsoft Project export
        baselines: {
          disabled: true,
        },
        dependencyEdit: true,
        filter: true,
        parentArea: {
          disabled: true,
        },
        progressLine: {
          disabled: true,
          statusDate: new Date(2019, 0, 25),
        },
        rollups: {
          disabled: true,
        },
        rowReorder: {
          showGrip: true,
        },
        timeRanges: {
          showCurrentTimeLine: true,
        },
        dependencies: {
          radius: 5,
        },
        taskMenu: {
          processItems({ taskRecord, items }) {
            configGantt.state.gantt = this;
          },
          items: {
            editTask: true,
            add: {
              menu: {
                // addTaskAbove: false,
                // subtask: false,
                // successor: false,
                // predecessor: false,
              },
            },
            deleteTask: {
              text: "Delete",
            },
            progress: {
              text: "Set progress",
              icon: "b-fa b-fa-percent",
              weight: 510,
              onItem: ({ item }) => {
                try {
                  if (configGantt.state.gantt.selectedRecords.length > 0) {
                    configGantt.state.gantt.selectedRecords.foreach(
                      (itemGantt) => {
                        itemGantt.setPercentDone(item.value);
                      }
                    );
                  } else {
                    Toast.show("Please select an item");
                  }
                } catch (ex) {
                  Toast.show("An error occured");
                }
              },
              menu: {
                items: [
                  {
                    icon: "b-fa b-fa-battery-empty",
                    text: "0%",
                    value: 0,
                  },
                  {
                    icon: "b-fa b-fa-battery-quarter",
                    text: "25%",
                    value: 25,
                  },
                  {
                    icon: "b-fa b-fa-battery-half",
                    text: "50%",
                    value: 50,
                  },
                  {
                    icon: "b-fa b-fa-battery-three-quarters",
                    text: "75%",
                    value: 75,
                  },
                  {
                    icon: "b-fa b-fa-battery-full",
                    text: "100%",
                    value: 100,
                  },
                ],
              },
            },
            baseline: {
              text: "Reset baseline",
              icon: "b-fa b-fa-eraser",
              weight: 511,
              onItem: ({ item }) => {
                const { value } = item;
                Toast.show("An error occured");
                try {
                  if (value === 0) {
                    configGantt.state.gantt.selectedRecords.forEach(
                      (itemGantt) => {
                        itemGantt.setBaseline(1);
                        itemGantt.setBaseline(2);
                        itemGantt.setBaseline(3);
                      }
                    );
                  } else {
                    configGantt.state.gantt.store.setBaseline(1);
                    configGantt.state.gantt.store.setBaseline(2);
                    configGantt.state.gantt.store.setBaseline(3);
                  }
                } catch (ex) {
                  Toast.show("An error occured");
                }
              },
              menu: {
                items: [
                  {
                    icon: "",
                    text: "Selected tasks",
                    value: 0,
                  },
                  {
                    icon: "",
                    text: "All tasks",
                    value: 1,
                  },
                ],
              },
            },
          },
        },
        taskEdit: {
          editorConfig: {
            autoClose: false,
          },
        },
        labels: {
          top: null,
          bottom: null,
          before: configGantt.label.beforeLabel,
          after: configGantt.label.afterLabel,
        },
      },
      tbar: {
        type: "gantttoolbar",
      },
      responsiveLevels: {
        small: {
          levelWidth: 600,
          rowHeight: 30,
          // barMargin: 5,
        },
        normal: {
          levelWidth: "*",
          rowHeight: 40,
          // barMargin: 10,
        },
      },
    },
    {
      type: "splitter",
    },
    {
      type: "resourcegrid",
      ref: "resourceGrid",
      store: project.resourceStore,
      width: "calc(15% - 7px)",
    },
  ],
};
