import { Popup, Toolbar, Toast, DateHelper } from "@bryntum/gantt";
import Drag from "./Drag";
let { configGantt } = require("./GanttCommon");

export default class GanttToolbar extends Toolbar {
  // Factorable type name
  static get type() {
    return "gantttoolbar";
  }

  static get $name() {
    return "GanttToolbar";
  }

  // Called when toolbar is added to the Gantt panel
  set parent(parent) {
    super.parent = parent;

    const me = this;

    me.gantt = parent;

    parent.project.on({
      load: "updateStartDateField",
      refresh: "updateStartDateField",
      thisObj: me,
    });

    me.styleNode = document.createElement("style");
    document.head.appendChild(me.styleNode);
  }

  get parent() {
    return super.parent;
  }
  static get configurable() {
    return {
      items: [
        /*Save & Reload butttongroup*/
        {
          type: "buttonGroup",
          toggleGroup: true,
          disabled: true,
          ref: "buttonGroupSaveReload",
          items: [
            {
              toggleGroup: "savereload",
              tooltip: "Save changes",
              icon: "b-fa b-fa-save",
              text: "Save",
              onAction: "up.onSaveChangesClick",
            },
            {
              toggleGroup: "savereload",
              cls: "noText",
              menu: {
                items: [
                  {
                    icon: "b-fa b-fa-sync",
                    text: "Reload (Discard changes)",
                    onItem: "up.onReloadButtonClick",
                  },
                ],
              },
            },
          ],
        },
        /*New Task*/
        {
          ref: "addTaskButton",
          icon: "b-fa b-fa-plus",
          text: "New",
          tooltip: "Create new task",
          onAction: "up.onAddTaskClick",
        },
        /*Undo-Redo group*/
        {
          type: "buttonGroup",
          items: [
            {
              ref: "undoRedo",
              type: "undoredo",
              items: {
                transactionsCombo: null,
                undoBtn: {
                  listeners: {
                    paint: "up.onUndoPaint",
                  },
                },
              },
            },
          ],
        },
        /*Indent-Outdent group*/
        {
          type: "buttonGroup",
          items: [
            {
              icon: "b-fa b-fa-indent",
              text: "Indent",
              tooltip: "Indent",
              onAction: "up.onIndentClick",
            },
            {
              icon: "b-fa b-fa-outdent",
              // text: "Outdent", /*Do not show text as this is rarely used button*/
              tooltip: "Outdent",
              onAction: "up.onOutdentClick",
            },
          ],
        },
        /*Collapse-Expand group*/
        {
          type: "buttonGroup",
          items: [
            {
              ref: "collapseAllButton",
              icon: "b-fa b-fa-angle-double-up",
              tooltip: "Collapse all",
              onAction: "up.onCollapseAllClick",
            },
            {
              ref: "expandAllButton",
              icon: "b-fa b-fa-angle-double-down",
              tooltip: "Expand all",
              onAction: "up.onExpandAllClick",
            },
          ],
        },
        /*ZoomIn-ZoomOut-ZoomToFit group*/
        {
          type: "buttonGroup",
          items: [
            {
              ref: "zoomInButton",
              icon: "b-fa b-fa-search-plus",
              tooltip: "Zoom in",
              onAction: "up.onZoomInClick",
            },
            {
              ref: "zoomOutButton",
              icon: "b-fa b-fa-search-minus",
              tooltip: "Zoom out",
              onAction: "up.onZoomOutClick",
            },
            {
              ref: "zoomToFitButton",
              icon: "b-fa b-fa-compress-arrows-alt",
              tooltip: "Zoom to fit",
              onAction: "up.onZoomToFitClick",
            },
            // {
            //   ref: "previousButton",
            //   icon: "b-fa b-fa-angle-left",
            //   tooltip: "Previous time span",
            //   onAction: "up.onShiftPreviousClick",
            // },
            // {
            //   ref: "nextButton",
            //   icon: "b-fa b-fa-angle-right",
            //   tooltip: "Next time span",
            //   onAction: "up.onShiftNextClick",
            // },
          ],
        },
        /*Progress*/
        {
          type: "button",
          ref: "percentageButton",
          icon: "b-fa b-fa-percent",
          text: "Progress",
          tooltip: "Set progress",
          toggleable: true,
          onItem: "up.onItemProgress",
          menu: {
            items: [
              {
                icon: "b-fa b-fa-battery-empty",
                text: "0%",
                value: 0,
              },
              {
                icon: "b-fa b-fa-battery-quarter",
                text: "25%",
                value: 25,
              },
              {
                icon: "b-fa b-fa-battery-half",
                text: "50%",
                value: 50,
              },
              {
                icon: "b-fa b-fa-battery-three-quarters",
                text: "75%",
                value: 75,
              },
              {
                icon: "b-fa b-fa-battery-full",
                text: "100%",
                value: 100,
              },
            ],
          },
        },
        /*View*/
        {
          type: "button",
          ref: "optionsButton",
          icon: "b-fa b-fa-tasks",
          text: "View",
          tooltip: "View options",
          toggleable: true,
          menu: {
            onItem: "up.onFeaturesClick",
            onBeforeShow: "up.onFeaturesShow",
            items: [
              {
                text: "Show critical path",
                checked: false,
                cls: "cls-critical-path",
                showCriticalPath: true,
              },
              {
                text: "Show dependencies",
                feature: "dependencies",
                checked: false,
              },
              {
                text: "Show task labels",
                checked: true,
                showLabelTask: true,
                labelsFeatureConfig: { before: configGantt.label.beforeLabel },
              },
              {
                text: "Show resource labels",
                checked: true,
                showLabelResources: true,
                labelsFeatureConfig: { after: configGantt.label.afterLabel },
              },
              {
                text: "Show project lines",
                feature: "projectLines",
                checked: false,
              },
              {
                text: "Highlight non-working time",
                feature: "nonWorkingTime",
                checked: false,
              },
              {
                text: "Enable cell editing",
                feature: "cellEdit",
                checked: false,
              },
              {
                text: "Show baselines",
                feature: "baselines",
                checked: false,
              },
              {
                text: "Show rollups",
                feature: "rollups",
                checked: false,
              },
              {
                text: "Show progress line",
                feature: "progressLine",
                checked: false,
              },
              {
                text: "Zoom to selection",
                //cls: "b-separator",          /*Hide top separator*/
                //cls: "b-separator-bottom",   /*Hide bottom separator*/
                scrollTask: "scrollTaskIntoViewOnCellClick",
                checked: false,
              },
              /*(EX) Hide this option as it is not very useful. It is easier to do on the main UI splitter*/
              //{
              //    text: "Hide schedule",
              //    cls: "b-separator-bottom",
              //    subGrid: "normal",
              //    checked: false,
              //},
              {
                type: "slider",
                ref: "rowHeight",
                text: "Row height",
                cls: "first",
                width: "12em",
                showValue: true,
                min: 30,
                max: 70,
                onInput: "up.onSettingsRowHeightChange",
              },
              // {
              //   type: "slider",
              //   ref: "barMargin",
              //   text: "Bar margin",
              //   cls: "last",
              //   width: "12em",
              //   showValue: true,
              //   min: 0,
              //   max: 10,
              //   onInput: "up.onSettingsMarginChange",
              // },
            ],
          },
        },
        /*othersButton*/
        {
          type: "button",
          ref: "othersButton",
          cls: "cls_other_button_gantt",
          icon: "b-fa b-fa-ellipsis-h",
          text: "",
          tooltip: "Other menus",
          toggleable: true,
          menu: {
            onItem: "up.onOtherButtonsClick",
            items: [
              /*Move-*/
              {
                ref: "moveButton",
                icon: "b-fa b-fa-exchange-alt",
                text: "Move project",
                tooltip: "Set project start date",
                toggleable: true,
                menu: {
                  type: "popup",
                  anchor: true,
                  cls: "movebtn-menu",
                  layoutStyle: {
                    flexDirection: "column",
                  },
                  items: [
                    {
                      type: "datefield",
                      ref: "startDateField",
                      label: "Project start",
                      // required  : true, (done on load)
                      listeners: {
                        change: "up.onStartDateChange",
                      },
                    },
                  ],
                },
              },
              {
                text: "Show resources pane",
                checked: false,
                ref: "showResourcepane",
              },
              {
                buttonConfig: {
                  icon: "b-fa b-fa-file-import",
                  text: "Import mpp/xml",
                },
                type: "filepicker",
                ref: "importMpp",
                fileFieldConfig: {
                  accept: ".mpp,.xml",
                },
                listeners: {
                  change: "up.onFileInputChange",
                  clear: () => {
                    try {
                      const { othersButton } = this._widgetMap;
                      othersButton.menu.items[0].clear();
                    } catch (ex) {}
                  },
                },
                cls: "btn-as-item",
              },
              {
                icon: "b-fa b-fa-file-export",
                text: "Export to mpp",
                ref: "exportToMpp",
              },
              {
                icon: "b-fa b-fa-file-export",
                text: "Export to xml",
                ref: "exportToXml",
              },
              {
                icon: "b-fa b-fa-save",
                text: "Save as template",
                ref: "saveAsTemplate",
              },
              {
                icon: "b-fa b-fa-image",
                text: "Save as image",
                ref: "saveAsImage",
              },
            ],
          },
        },
        "->",
        /*Search input */
        {
          type: "textfield",
          ref: "filterByName",
          cls: "filter-by-name",
          flex: "0 0 12em",
          placeholder: "Search",
          clearable: true,
          keyStrokeChangeDelay: 100,
          triggers: {
            filter: {
              align: "start",
              cls: "b-fa b-fa-search",
              // clearable: true
            },
          },
          onChange: "up.onFilterChange",
        },
      ],
    };
  }

  updateStartDateField() {
    const { othersButton } = this.widgetMap;
    othersButton.menu.items[0].menu.items[0].value =
      this.gantt.project.startDate;
    // This handler is called on project.load/propagationComplete, so now we have the
    // initial start date. Prior to this time, the empty (default) value would be
    // flagged as invalid.
    othersButton.menu.items[0].menu.items[0].required = true;
  }

  // region controller methods
  onSaveChangesClick() {
    Toast.show("Request missing");
    //to do
  }

  onReloadButtonClick() {
    const popup = new Popup({
      header: "Discard Changes",
      modal: {
        closeOnMaskTap: true,
      },
      closeAction: "destroy",
      bbar: [
        {
          text: "Yes",
          minWidth: 100,
          cls: "b-raised b-blue",
          onAction: () => {
            const { gantt } = this;
            var b = gantt._store.crudManager;
            b.load();
            const itemSave = gantt.tbar.items.find((obj) => {
              return obj.initialConfig.ref === "buttonGroupSaveReload";
            });
            if (itemSave != null) {
              itemSave.disable();
            }
            configGantt.state.isLoaded = false;
            Toast.show("Changes discarded");
            popup.close();
          },
        },
        {
          text: "No",
          minWidth: 100,
          onAction: "up.close",
        },
      ],
      html: `
      <p style="line-height:1.5em">Are you sure you want to discard changes?</p>
      `,
    });
  }

  async onAddTaskClick() {
    const { gantt } = this,
      added = gantt.taskStore.rootNode.appendChild({
        name: "New task",
        duration: 1,
      });

    // run propagation to calculate new task fields
    await gantt.project.propagateAsync();

    // scroll to the added task
    await gantt.scrollRowIntoView(added);

    gantt.features.cellEdit.startEditing({
      record: added,
      field: "name",
    });
  }

  onEditTaskClick() {
    const { gantt } = this;

    if (gantt.selectedRecord) {
      gantt.editTask(gantt.selectedRecord);
    } else {
      Toast.show("Please select the task you want to edit");
    }
  }

  onExpandAllClick() {
    this.gantt.expandAll();
  }

  onCollapseAllClick() {
    this.gantt.collapseAll();
  }

  onZoomInClick() {
    this.gantt.zoomIn();
  }

  onZoomOutClick() {
    this.gantt.zoomOut();
  }

  onZoomToFitClick() {
    this.gantt.zoomToFit({
      leftMargin: 50,
      rightMargin: 50,
    });
  }

  onIndentClick() {
    this.gantt.taskStore.indent(this.gantt.selectedRecords);
  }

  onOutdentClick() {
    this.gantt.taskStore.outdent(this.gantt.selectedRecords);
  }

  // onShiftPreviousClick() {
  //   this.gantt.shiftPrevious();
  // }

  // onShiftNextClick() {
  //   this.gantt.shiftNext();
  // }

  onStartDateChange({ value, oldValue }) {
    if (!oldValue) {
      // ignore initial set
      return;
    }

    this.gantt.startDate = DateHelper.add(value, -1, "week");

    this.gantt.project.setStartDate(value);
  }

  onOtherButtonsClick({ source: item }) {
    const { gantt } = this;
    if (item._ref === "exportToMpp") {
      const filename =
        gantt.project.taskStore.first &&
        `${gantt.project.taskStore.first.name}.mpp`;
      gantt.features.mspExport.export({
        filename,
      });
    } else if (item._ref === "exportToXml") {
      const filename =
        gantt.project.taskStore.first &&
        `${gantt.project.taskStore.first.name}.xml`;
      gantt.features.mspExport.export({
        filename,
      });
    } else if (item._ref === "saveAsImage") {
      gantt.features.pdfExport.showExportDialog();
    } else if (item._ref === "saveAsTemplate") {
      Toast.show("You have clicked save as template");
    } else if (item._ref === "showResourcepane") {
      let resourceGrid = this.getWidgetByRef("resourceGrid");
      item.checked ? (gantt.width = "85%") : (gantt.width = "100%");
      resourceGrid.hidden = !item.checked;
    }
  }

  onFilterChange({ value }) {
    if (value === "") {
      this.gantt.taskStore.clearFilters();
    } else {
      value = value.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");

      this.gantt.taskStore.filter({
        filters: (task) => task.name && task.name.match(new RegExp(value, "i")),
        replace: true,
      });
    }
  }

  onFeaturesClick({ source: item }) {
    const { gantt } = this;
    if (item.feature) {
      const feature = gantt.features[item.feature];
      feature.disabled = !feature.disabled;
    } else if (item.subGrid) {
      const subGrid = gantt.subGrids[item.subGrid];
      subGrid.collapsed = !subGrid.collapsed;
    } else if (item.scrollTask) {
      gantt.scrollTaskIntoViewOnCellClick =
        !gantt.scrollTaskIntoViewOnCellClick;
    } else if (item.showLabelTask) {
      if (item.checked) {
        gantt.features.labels.setConfig({
          before: configGantt.label.beforeLabel,
        });
      } else {
        gantt.features.labels.setConfig({ before: null });
      }
      gantt.refresh(true);
    } else if (item.showLabelResources) {
      if (item.checked) {
        gantt.features.labels.setConfig({
          after: configGantt.label.afterLabel,
        });
      } else {
        gantt.features.labels.setConfig({ after: null });
      }
      gantt.refresh(true);
    } else if (item.showCriticalPath) {
      gantt.features.criticalPaths.disabled = !item.checked;
    }
  }

  onFeaturesShow({ source: menu }) {
    try {
      const { gantt } = this;
      menu.items.map((item) => {
        const {
          feature,
          scrollTask,
          onInput,
          showLabelTask,
          showLabelResources,
          showCriticalPath,
        } = item;
        if (feature) {
          // a feature might be not presented in the gantt
          // (the code is shared between "advanced" and "php" demos which use a bit different set of features)
          if (gantt.features[feature]) {
            item.checked = !gantt.features[feature].disabled;
          }
          // hide not existing features
          else {
            item.hide();
          }
        } else {
          if (scrollTask) {
            item.checked = gantt.scrollTaskIntoViewOnCellClick;
          } else if (onInput) {
            //To set default value etc
          } else if (showLabelTask || showLabelResources) {
            //To set default value etc
          } else if (showCriticalPath) {
            //To set default value etc
          } else {
            item.checked = gantt.subGrids[item.subGrid].collapsed;
          }
        }
        return item;
      });
    } catch (ex) {
      //ignore ex
    }
  }

  onSettingsShow({ source: menu }) {
    const { gantt } = this,
      { rowHeight } = menu.widgetMap;

    rowHeight.value = gantt.rowHeight;
    // barMargin.value = gantt.barMargin;
    // barMargin.max = gantt.rowHeight / 2 - 5;
  }

  onSettingsRowHeightChange({ value }) {
    this.gantt.rowHeight = value;
    this.widgetMap.settingsButton.menu.widgetMap.barMargin.max = value / 2 - 5;
  }

  // onSettingsMarginChange({ value }) {
  //   this.gantt.barMargin = value;
  // }
  onUndoPaint({ value }) {
    configGantt.state.gantt = this.gantt;
    configGantt.state.resourceGrid = this.getWidgetByRef("resourceGrid");
    const othersButton = configGantt.state.gantt.tbar.items.find((obj) => {
      return obj.initialConfig.ref === "othersButton";
    });
    if (othersButton) {
      const showResourcepane = othersButton.menu.items.find((obj) => {
        return obj.initialConfig.ref === "showResourcepane";
      });
      if (showResourcepane) {
        showResourcepane.checked
          ? (configGantt.state.gantt.width = "85%")
          : (configGantt.state.gantt.width = "100%");
        configGantt.state.resourceGrid.hidden = !showResourcepane.checked;
      }
      new Drag({
        grid: configGantt.state.resourceGrid,
        gantt: configGantt.state.gantt,
        constrain: false,
      });
    }
  }

  onItemProgress({ item }) {
    try {
      const { gantt } = this;
      if (gantt.selectedRecords.length > 0) {
        gantt.selectedRecords.foreach((itemGantt) => {
          itemGantt.setPercentDone(item.value);
        });
      } else {
        Toast.show("Please select an item");
      }
    } catch (ex) {
      Toast.show("Please select an item");
    }
  }
  // endregion
}
// Register this widget type with its Factory
GanttToolbar.initClass();
