import { TreeGrid, AvatarRendering } from "@bryntum/gantt";
let { configGantt } = require("./GanttCommon");

export default class ResourceGrid extends TreeGrid {
  static get $name() {
    return "ResourceGrid";
  }
  static get type() {
    return "resourcegrid";
  }
  static get configurable() {
    return {
      resourceImagePath: "/images/users",
      selectionMode: {
        row: true,
        multiSelect: true,
      },
      features: {
        columnReorder: false,
        headerMenu: false,
        cellEdit: false,
      },
      columns: [
        {
          ref: "closeResourcesPanelButton",
          icon: "b-fa b-fa-times",
          text: "",
          cls: "closeBtn",
          type: "action",
          tooltip: "Close Resources Panel",
          sortable() {
            if (!configGantt.state.resourceGrid.hidden) {
              const othersButton = configGantt.state.gantt.tbar.items.find(
                (obj) => {
                  return obj.initialConfig.ref === "othersButton";
                }
              );
              if (othersButton) {
                const showResourcepane = othersButton.menu.items.find((obj) => {
                  return obj.initialConfig.ref === "showResourcepane";
                });
                if (showResourcepane) {
                  showResourcepane.checked = false;
                  configGantt.state.gantt.width = "100%";
                  configGantt.state.resourceGrid.hidden = true;
                }
              }
            }
          },
        },
        {
          type: "tree",
          text: "Resources",
          showEventCount: false,
          field: "name",
          renderer: ({ record, grid, value }) => ({
            class: "b-resource-info",
            children: [
              grid.avatarRendering.getResourceAvatar({
                initials: record.initials,
                color: record.eventColor,
                iconCls: record.iconCls,
                imageUrl: record.image
                  ? `${grid.resourceImagePath}${record.image}`
                  : null,
              }),
              value,
            ],
          }),
          flex: 1,
        },
      ],
    };
  }

  afterConstruct() {
    this.avatarRendering = new AvatarRendering({
      element: this.element,
    });
  }
}
ResourceGrid.initClass();
